/***************************************************************************
 *   Copyright (C) 2005 by Florian Roth   *
 *   florian@synatic.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "metabar_widget.h"

#include <qsize.h>
#include <qfont.h>
#include <qcolor.h>
#include <qcursor.h>
#include <qhbox.h>
#include <qfile.h>
#include <qobjectlist.h>
#include <qtooltip.h>

#include <klocale.h>
#include <kiconloader.h>
#include <ktrader.h>
#include <kmimetype.h>
#include <krun.h>
#include <kurl.h>
#include <kglobalsettings.h>
#include <kpropertiesdialog.h>
#include <kapplication.h>
#include <dcopref.h>
#include <dcopclient.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kio/jobclasses.h>
#include <kmessagebox.h>
#include <ksycocaentry.h>
#include <kaction.h>

#include <soundserver.h>
#include <kartsdispatcher.h>

#include <kdebug.h>

MetabarWidget::MetabarWidget(QWidget *parent, const char *name)
    : QWidget( parent, name )
{
  playobj = 0;
  preview_job = 0;
  
  show_more = 0;
  
  player_timer = new QTimer();
  connect(player_timer, SIGNAL(timeout()), this, SLOT(updatePlayerStatus()));
  
  dispatcher = new KArtsDispatcher();
  soundServer = Arts::Reference("global:Arts_SoundServerV2");
  factory = new KDE::PlayObjectFactory(soundServer);
  
  config = new KSimpleConfig("metabarrc");
  
  QColor background = KGlobalSettings::baseColor();
  if(!background.isValid()){
    background = QColor(255, 255, 255);
  }
  
  currentPath = new QString("");
  
  actionList.setAutoDelete(true);
  linkList.setAutoDelete(true);
  infoList.setAutoDelete(true);
  infoDict.setAutoDelete(true);
    
  dir_watch = new KDirWatch();
  connect(dir_watch, SIGNAL(dirty(const QString&)), this, SLOT(slotUpdateCurrentInfo(const QString&)));
  connect(dir_watch, SIGNAL(created(const QString&)), this, SLOT(slotUpdateCurrentInfo(const QString&)));
  connect(dir_watch, SIGNAL(deleted(const QString&)), this, SLOT(slotDeleteCurrentInfo(const QString&)));
    
  //top box  
  QFrame *top_frame = new QFrame(this);
  top_frame->setMargin(10);
  top_frame->setPaletteBackgroundColor(background);
  top_frame->setFrameShape(QFrame::StyledPanel);
  top_frame->setFrameShadow(QFrame::Raised);
  
  config_button = new KPushButton(top_frame);
  config_button->setPixmap(SmallIcon("configure"));
  config_button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
  connect(config_button, SIGNAL(clicked()), this, SLOT(slotShowConfigDialog()));
  
  icon = new KURLLabel(top_frame);
  connect(icon, SIGNAL(leftClickedURL()), this, SLOT(slotShowPropertiesDialog()));
  
  name_label = new KSqueezedTextLabel(top_frame);
  QFont font = name_label->font();
  font.setBold(true);
  font.setPointSize(14);
  name_label->setFont(font);
  
  type_label = new KSqueezedTextLabel(top_frame);
  
  QGridLayout *top_frame_layout = new QGridLayout(top_frame, 2, 3, 5, 5);
  
  top_frame_layout->addMultiCellWidget(icon, 0, 1, 0, 0);
  top_frame_layout->addWidget(name_label, 0, 1);
  top_frame_layout->addWidget(config_button, 0, 2);
  top_frame_layout->addMultiCellWidget(type_label, 1, 1, 1, 2);
  
  //action box
  action_frame = new QFrame(this);
  action_frame->setPaletteBackgroundColor(background);
  action_frame->setFrameShape(QFrame::StyledPanel);
  action_frame->setFrameShadow(QFrame::Raised);
  
  action_title = new KPushButton(action_frame);
  action_title->setToggleButton(TRUE);
  connect(action_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  
  font = action_title->font();
  font.setBold(true);
  action_title->setFont(font);
  
  action_frame_layout = new QVBoxLayout(action_frame, 5, 5);
  action_frame_layout->addWidget(action_title);

  //link box  
  link_frame = new QFrame(this);
  link_frame->setPaletteBackgroundColor(background);
  link_frame->setFrameShape(QFrame::StyledPanel);
  link_frame->setFrameShadow(QFrame::Raised);
  
  link_title = new KPushButton(i18n("Links"), link_frame);
  link_title->setToggleButton(TRUE);  
  font = link_title->font();
  font.setBold(true);
  link_title->setFont(font);
  connect(link_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  
  link_frame_layout = new QVBoxLayout(link_frame, 5, 5);
  link_frame_layout->addWidget(link_title);
  
  loadLinkList();
  
  //info box  
  info_frame = new QFrame(this);
  info_frame->setPaletteBackgroundColor(background);
  info_frame->setFrameShape(QFrame::StyledPanel);
  info_frame->setFrameShadow(QFrame::Raised);
  
  info_title = new KPushButton(i18n("Information"), info_frame);
  info_title->setToggleButton(TRUE);
  connect(info_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  
  font = info_title->font();
  font.setBold(true);
  info_title->setFont(font);
  
  info_table = new QLabel("", info_frame);
  
  info_frame_layout = new QVBoxLayout(info_frame, 5, 5);
  info_frame_layout->addWidget(info_title);
  info_frame_layout->addWidget(info_table);
  
  //preview box  
  preview_frame = new QFrame(this);
  preview_frame->setPaletteBackgroundColor(background);
  preview_frame->setFrameShape(QFrame::StyledPanel);
  preview_frame->setFrameShadow(QFrame::Raised);
  
  preview_title = new KPushButton(i18n("Preview"), preview_frame);
  preview_title->setToggleButton(TRUE);
  connect(preview_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  connect(preview_title, SIGNAL(toggled(bool)), this, SLOT(slotCreatePreview(bool)));
  
  font = info_title->font();
  font.setBold(true);
  preview_title->setFont(font);
  
  preview = new QLabel(0);
  preview->setAlignment(Qt::AlignHCenter);
  
  player_frame = new QFrame(preview_frame);
  
  player_start = new KPushButton(player_frame);
  player_start->setPixmap(SmallIcon("player_play"));
  connect(player_start, SIGNAL(clicked()), this, SLOT(startPlayer()));
  
  player_pause = new KPushButton(player_frame);
  player_pause->setPixmap(SmallIcon("player_pause"));
  connect(player_pause, SIGNAL(clicked()), this, SLOT(pausePlayer()));
  
  player_stop = new KPushButton(player_frame);
  player_stop->setPixmap(SmallIcon("player_stop"));
  connect(player_stop, SIGNAL(clicked()), this, SLOT(stopPlayer()));
  
  player_status = new QSlider(Qt::Horizontal, player_frame);
  player_status->setTracking(FALSE);
  connect(player_status, SIGNAL(valueChanged(int)), this, SLOT(seekPlayer(int)));
  
  video = new KVideoWidget();
  
  preview_stack = new QWidgetStack(preview_frame);
  preview_stack->addWidget(preview);
  preview_stack->addWidget(video);
  preview_stack->raiseWidget(preview);
  
  QHBoxLayout *player_frame_layout2 = new QHBoxLayout();
  player_frame_layout2->addWidget(player_start);
  player_frame_layout2->addWidget(player_pause);
  player_frame_layout2->addWidget(player_stop);
  
  QVBoxLayout *player_frame_layout = new QVBoxLayout(player_frame, 5, 5);
  player_frame_layout->addLayout(player_frame_layout2);
  player_frame_layout->addWidget(player_status);
  
  QVBoxLayout *preview_frame_layout = new QVBoxLayout(preview_frame, 5, 5);
  preview_frame_layout->addWidget(preview_title);
  preview_frame_layout->addWidget(preview_stack);
  preview_frame_layout->addWidget(player_frame);
  
  //main layout  
  QVBoxLayout *main_layout = new QVBoxLayout(this, 5, 5);
  main_layout->addWidget(top_frame);
  main_layout->addWidget(action_frame);
  main_layout->addWidget(link_frame);
  main_layout->addWidget(info_frame);
  main_layout->addWidget(preview_frame);
  main_layout->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Expanding));
  
  //load config
  loadConfig();
}

MetabarWidget::~MetabarWidget(){
  config->setGroup("General");
  config->writeEntry("CollapseActions", action_title->isOn());
  config->writeEntry("CollapseInfo", info_title->isOn());
  config->writeEntry("CollapsePreview", preview_title->isOn());
  config->writeEntry("CollapseLinks", link_title->isOn());
  config->sync();
  
  deletePlayer();
  
  delete playobj;
  delete factory;
  delete player_timer;
  delete preview_job;
  
  delete config;
  delete dir_watch;
  delete currentPath;
  delete currentItems;
}

void MetabarWidget::setFileItems(const KFileItemList &items)
{   
  int count = items.count();
  
  if(count == 0){  
    currentItems = 0;
    
    QString url = getCurrentFolder();
    QString path = KURL(url).path();
    
    if(KURL(url) != KURL(*currentPath)){
      KFileItem current(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      
      if(dir_watch->contains(*currentPath)){
        if(current.isDir()){
          dir_watch->removeDir(*currentPath);
        }
        else{
          dir_watch->removeFile(*currentPath);
        }
      }
      
      const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(url), true);
      
      if(item.isDir()){
        dir_watch->addDir(item.url().path());
      }
      else{
        dir_watch->addFile(item.url().path());
      }
          
      *currentPath = path;     
      
      skip_delete = TRUE;
      updateSingleInfo(item);
    }
  }
  
  else if(count == 1){
    currentItems = 0;
    
    if(items.getFirst()->url() != KURL("") && items.getFirst()->url() != KURL(*currentPath)){
      KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      
      if(dir_watch->contains(*currentPath)){
        if(item.isDir()){
          dir_watch->removeDir(*currentPath);
        }
        else{
          dir_watch->removeFile(*currentPath);
        }
      }
      
      KFileItem *newItem = items.getFirst();
      
      if(newItem->url().path() == "blank"){
        newItem = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(getCurrentFolder()), true);
      }
      
      if(newItem->isDir()){
        dir_watch->addDir(newItem->url().path());
      }
      else{
        dir_watch->addFile(newItem->url().path());
      }      
      
      *currentPath = newItem->url().path();      
      
      skip_delete = FALSE;
      updateSingleInfo(*newItem);
    }
  }
  else{
    *currentPath = "";
    
    if(!currentItems || *currentItems != items){
      currentItems = new KFileItemList(items);
      updateMultiInfo(items);
    }
  }
}

void MetabarWidget::updateSingleInfo(const KFileItem &item){
  setUpdatesEnabled(FALSE);
  
  single_selection = TRUE;
  
  //delete old data
  actionList.clear();
  infoDict.clear();
  infoList.clear();
  runlist.clear();
  
  deletePlayer();
  
  show_more = 0;
  
  //get new info  
  const KURL url = item.url();
  QString name = item.name();
  name = name.isEmpty() ? i18n("Root Folder") : name;
  
  name_label->setText(name);

  KFileItem *that = const_cast<KFileItem *>(&item);
  type_label->setText(that->determineMimeType()->comment());
  
  icon->setPixmap(item.pixmap(0, KIcon::DefaultState));
  
  if(item.isDir()){
    action_title->setText(i18n("Actions"));
    
    if(linkList.count() > 0){
      link_frame->show();
    }
    else if(link_frame->isVisible()){
      link_frame->hide();
    }
    
    //create folder
    LinkWidget *createfolder_box = new LinkWidget(action_frame);
    createfolder_box->setText(i18n("Create New Folder"));
    createfolder_box->setURL("konq_create_dir");
    createfolder_box->setPixmap(SmallIcon("folder_new"));
    
    connect(createfolder_box, SIGNAL(leftClickedURL(const QString&)), this, SLOT(callAction(const QString&)));    
        
    if(!action_title->isOn()){
      createfolder_box->show();
    }
        
    action_frame_layout->add(createfolder_box);
    actionList.append(createfolder_box);
    
    //share
    LinkWidget *share_box = new LinkWidget(action_frame);
    share_box->setText(i18n("Share"));
    share_box->setURL("exec://share");
    share_box->setPixmap(SmallIcon("network"));
    
    connect(share_box, SIGNAL(leftClickedURL()), this, SLOT(slotShowSharingDialog()));   
        
    if(!action_title->isOn()){
      share_box->show();
    }
        
    action_frame_layout->add(share_box);
    actionList.append(share_box);
    
    //rename
    LinkWidget *rename_box = new LinkWidget(action_frame);
    rename_box->setText(i18n("Rename"));
    rename_box->setURL("rename");
    rename_box->setPixmap(SmallIcon("edit"));
    
    connect(rename_box, SIGNAL(leftClickedURL(const QString&)), this, SLOT(callAction(const QString&)));
        
    if(!action_title->isOn()){
      rename_box->show();
    }
        
    action_frame_layout->add(rename_box);
    actionList.append(rename_box);
    
    //delete
    if(!skip_delete){
      LinkWidget *delete_box = new LinkWidget(action_frame);
      delete_box->setText(i18n("Delete"));
      delete_box->setURL("del");
      delete_box->setPixmap(SmallIcon("editdelete"));
      
      connect(delete_box, SIGNAL(leftClickedURL(const QString&)), this, SLOT(callAction(const QString&)));   
          
      if(!action_title->isOn()){
        delete_box->show();
      }
          
      action_frame_layout->add(delete_box);
      actionList.append(delete_box);
    }
        
    //hide preview
    preview_frame->hide();
  }
  else{
    action_title->setText(i18n("Open with %1").arg("..."));
    link_frame->hide();
    
    KTrader::OfferList offers;
    offers = KTrader::self()->query(item.mimetype(), "Type == 'Application' and DesktopEntryName != 'kfmclient' and DesktopEntryName != 'kfmclient_dir' and DesktopEntryName != 'kfmclient_html'");
    if(!offers.isEmpty()){
      int id = 0;
      
      KTrader::OfferList::ConstIterator it = offers.begin();
      config->setGroup("General");
      
      for(; it != offers.end(); it++){        
        QCString nam;
        nam.setNum(id);        
        LinkWidget *url_box = new LinkWidget(action_frame, nam.prepend("action_"));
        url_box->setText((*it)->name());
        url_box->setURL(url.url());
        url_box->setPixmap((*it)->pixmap(KIcon::Small));
        
        connect(url_box, SIGNAL(leftClickedURL()), this, SLOT(slotRun()));  
        
        if(id < config->readNumEntry("MaxEntries", 3) && !action_title->isOn()){
          url_box->show();
        }
        
        runlist[id++] = *it;
        
        action_frame_layout->add(url_box);
        actionList.append(url_box);
      }
      
      if(id > config->readNumEntry("MaxEntries", 3)){
        show_more = new KURLLabel(action_frame);
        show_more->setURL("more");
        show_more->setPixmap(SmallIcon("1downarrow"));
        show_more->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        show_more->setAlignment(Qt::AlignRight);
        
        connect(show_more, SIGNAL(leftClickedURL()), this, SLOT(slotShowMore()));
        
        if(!action_title->isOn()){
          show_more->show();
        }
        
        action_frame_layout->add(show_more);
        actionList.append(show_more);
      }
    }
    
    if(!preview_frame->isVisible()){
      preview_frame->show();
    }
  }
  
  QString info("<table></td></tr><tr><td>" + i18n("Size:") + "</td><td>");
  info = info.append(KIO::convertSize(item.size()));
  info = info.append("</td></tr><tr><td>" + i18n("User:") + "</td><td>");
  info = info.append(item.user());
  info = info.append("</td></tr><tr><td>" + i18n("Group:") + "</td><td>");
  info = info.append(item.group());
  info = info.append("</td></tr><tr><td>" + i18n("Permissions:") + "</td><td>");
  info = info.append(item.permissionsString());
  info = info.append("</td></tr><tr><td>" + i18n("Last modified:") + "</td><td>");
  info = info.append(item.timeString(KIO::UDS_MODIFICATION_TIME));
  info = info.append("</td></tr><tr><td>" + i18n("Accessed:") + "</td><td>");
  info = info.append(item.timeString(KIO::UDS_ACCESS_TIME));
  
  if(item.isLink()){
    info = info.append("<tr><td>" + i18n("Linktarget:") + "</td><td>");
    info = info.append(item.linkDest());
    info = info.append("</td></tr>");
  }
  
  info = info.append("</table>");
  info_table->setText(info);
  
  if(!item.isDir()){
    const KFileMetaInfo &metaInfo = item.metaInfo();
    if(metaInfo.isValid()){
      QStringList groups = metaInfo.supportedGroups();
      
      for(QStringList::ConstIterator it = groups.begin(); it != groups.end(); ++it){
        KFileMetaInfoGroup group = metaInfo.group(*it);
        if(group.isValid()){
          KURLLabel *title_label = new KURLLabel("exec://" + group.name(), group.translatedName(), info_frame);
          title_label->setUnderline(FALSE);
          connect(title_label, SIGNAL(leftClickedURL()), this, SLOT(slotPopupInfo()));
          
          QLabel *info_label = new QLabel(info_frame);
          
          infoDict.insert(group.translatedName(), info_label);
          infoList.append(title_label);
          
          info_frame_layout->addWidget(title_label);
          info_frame_layout->addWidget(info_label);
          
          QStringList keys = group.supportedKeys();
          
          QString str("<table>");
        
          for(QStringList::ConstIterator it = keys.begin(); it != keys.end(); ++it){
            const KFileMetaInfoItem metaInfoItem = group.item(*it);
            
            if(metaInfoItem.isValid()){
              str = str.append("<tr><td>" + metaInfoItem.translatedKey() + "</td><td>");
              str = str.append(metaInfoItem.string());
              str = str.append("</td></tr>");
            }
          }
          str = str.append("</table>");
          info_label->setText(str);
          QToolTip::add(info_label, str);
          
          if(!info_title->isOn()){
            title_label->show();
          }
        }
      }
    }
  }
  
  if(!preview_title->isOn()){
    killJobs();
    
    if(item.mimetype().startsWith("video/") || item.mimetype().startsWith("audio/")){
      player_status->setValue(0);
      player_frame->show();
      
      playobj = factory->createPlayObject(item.url(), true);

      if (playobj && !playobj->object().isNull()){
        if(item.mimetype().startsWith("video/")){
          isVideo = TRUE;
          video->embed((Arts::VideoPlayObject)Arts::DynamicCast(playobj->object()));
        }
        else{
          isVideo = FALSE;
        }
      }
    }
    else{
      player_frame->hide();
    }
        
    preview->clear();
    preview->setText(i18n("Creating preview"));
    
    preview_stack->raiseWidget(preview);
    preview_stack->show();

    preview_job = KIO::filePreview(KURL::List(item.url()), preview_stack->width(), 0, 0, 0, TRUE, TRUE, 0);
    
    connect(preview_job, SIGNAL(gotPreview(const KFileItem*, const QPixmap&)), this, SLOT(slotSetPreview(const KFileItem*, const QPixmap&)));
    connect(preview_job, SIGNAL(result(KIO::Job*)), this, SLOT(slotGotPreview()));
  }
  
  setUpdatesEnabled(TRUE);
  repaint();
}

void MetabarWidget::updateMultiInfo(const KFileItemList &items){
  setUpdatesEnabled(FALSE);
  
  single_selection = FALSE;
  
  //delete old data
  actionList.clear();
  infoDict.clear();
  infoList.clear();
  runlist.clear();
  
  deletePlayer();
  
  show_more = 0;
  
  //get new info  
  action_title->setText(i18n("Actions"));
  
  if(linkList.count() > 0){
    link_frame->show();
  }
  else if(link_frame->isVisible()){
    link_frame->hide();
  }
  
  KIO::filesize_t size = 0;
  int files = 0;
  int dirs  = 0;
  
  for(KFileItemListIterator it(items); it.current(); ++it){
    size += (*it)->size();
    
    if((*it)->isDir()){
      dirs++;
    }
    else{
      files++;
    }
  }
  
  QString name = QString().setNum(items.count()) + " " + i18n("Items");  
  name_label->setText(name);
  
  type_label->setText(QString().setNum(files) + " " + i18n("Files") + ", " + QString().setNum(dirs) + " " + i18n("Folders"));
  
  icon->setPixmap(DesktopIcon("kmultiple"));
  
  LinkWidget *delete_box = new LinkWidget(action_frame);
  delete_box->setText(i18n("Delete"));
  delete_box->setURL("del");
  delete_box->setPixmap(SmallIcon("editdelete"));
  
  connect(delete_box, SIGNAL(leftClickedURL(const QString&)), this, SLOT(callAction(const QString&)));   
        
  if(!action_title->isOn()){
    delete_box->show();
  }

  action_frame_layout->add(delete_box);
  actionList.append(delete_box);  
  
  QString info("<table><tr><td>" + i18n("Size:") + "</td><td>");
  info = info.append(KIO::convertSize(size));
  info = info.append("</td></tr></table>");
  
  info_table->setText(info);
  
  killJobs();
  preview->clear();
  preview_stack->hide();
  preview_frame->hide();
  
  setUpdatesEnabled(TRUE);
  repaint();
}

void MetabarWidget::slotUpdateCurrentInfo(const QString &path)
{
  if(single_selection){
    const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(path), true);
    updateSingleInfo(item);
  }
}

void MetabarWidget::slotDeleteCurrentInfo(const QString&)
{
  if(single_selection){
    QString url = getCurrentFolder();
    
    if(KURL(url) != KURL(*currentPath)){
      if(dir_watch->contains(*currentPath)){
        dir_watch->removeDir(*currentPath);
      }
      dir_watch->addDir(url);
          
      *currentPath = url;
          
      const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url, true);
        
      updateSingleInfo(item);
    }
  }
}

void MetabarWidget::slotGotPreview()
{
  preview_job = 0;
  
  if(!preview_title->isOn()){
    if (preview->pixmap()){
      preview_stack->raiseWidget(preview);
      preview_stack->show();
    }
    else{
      preview->clear();
      preview_stack->hide();
    }
  }
}

void MetabarWidget::killJobs()
{
  if(preview_job){
    preview_job->kill();
    preview_job = 0;
  }
}

void MetabarWidget::slotSetPreview(const KFileItem*, const QPixmap& icon)
{
  preview_stack->resize(icon.size());
  preview_stack->setMaximumHeight(icon.height());
  
  preview->setPixmap(icon);
  
  video->setFixedSize(icon.size());
}

void MetabarWidget::slotCreatePreview(bool toggled)
{
  if(!toggled){
    killJobs();
    
    const KURL url(*currentPath);
    KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url, true);
    
    if(!playobj){
      if(item.mimetype().startsWith("video/") || item.mimetype().startsWith("audio/")){
        player_frame->show();
        
        playobj = factory->createPlayObject(item.url(), true);
  
        if (playobj && !playobj->object().isNull()){
          if(item.mimetype().startsWith("video/")){
            isVideo = TRUE;
            video->embed((Arts::VideoPlayObject)Arts::DynamicCast(playobj->object()));
          }
          else{
            isVideo = FALSE;
          }
        }
      }
      else{
        player_frame->hide();
      }
    }
        
    preview->clear();
    preview->setText(i18n("Creating preview"));
      
    preview_stack->raiseWidget(preview);
    preview_stack->show();
  
    preview_job = KIO::filePreview(KURL::List(url), preview_stack->width(), 0, 0, 0, TRUE, FALSE, 0);
    
    connect(preview_job, SIGNAL(gotPreview(const KFileItem*, const QPixmap&)), this, SLOT(slotSetPreview(const KFileItem*, const QPixmap&)));
    connect(preview_job, SIGNAL(result(KIO::Job*)),this, SLOT(slotGotPreview()));
  }
}

void MetabarWidget::slotRun()
{
  const QObject *sender = this->sender();
  if(sender->inherits("LinkWidget")){
    const LinkWidget *label = static_cast<const LinkWidget*>(sender);
    
    QCString sender_name = sender->name();
    int id = sender_name.mid(sender_name.find("_") + 1).toInt();
      
    QMap<int,KService::Ptr>::Iterator it = runlist.find(id);
    if(it != runlist.end()){
      KRun::run( **it, KURL::List(KURL(label->url())));
      return; 
    }
  }
}

void MetabarWidget::slotExpand(bool collapse)
{
  const QObject *sender = this->sender();
  const QWidget *sender_widget = static_cast<const QWidget*>(sender);
  
  if(sender_widget->parentWidget() == action_frame){
    bool stop = FALSE;
  
    if(show_more){
      stop = show_more->url() == i18n("more");
      
      if(stop && !collapse){
        show_more->show();
      }
    }
  
    QWidget *widget;
    for(widget = actionList.first(); widget; widget = actionList.next()){
      if(collapse){
        widget->hide();
      }
      else{
        config->setGroup("General");
        if(!stop || actionList.at() < config->readNumEntry("MaxEntries", 3)){
          widget->show();
        }
      }
    }
  }
  else{
    QLayoutIterator it = sender_widget->parentWidget()->layout()->iterator();
    QLayoutItem *child;
    
    while((child = it.current()) != 0){
      QWidget *widget = child->widget();
      
      if(widget && !widget->inherits("KPushButton")){      
        if(collapse){
          widget->hide();
        }
        else{
          widget->show();
        }
      }
      ++it;
    }
  }
}

void MetabarWidget::slotShowMore()
{
  if(show_more){
    bool show = show_more->url() == i18n("more");
    
    if(show){
      show_more->setPixmap(SmallIcon("1uparrow"));
      show_more->setURL("less");
    }
    else{
      show_more->setPixmap(SmallIcon("1downarrow"));
      show_more->setURL("more");
    }
  
    config->setGroup("General");
    for(uint i = config->readNumEntry("MaxEntries", 3); i < actionList.count() - 1; i++){
      if(show){
        actionList.at(i)->show();
      }
      else{
        actionList.at(i)->hide();
      }
    }
  }
}

void MetabarWidget::slotShowPropertiesDialog()
{
  if(single_selection){
    if(!currentPath->isNull() && !currentPath->isEmpty()){
      KFileItem *item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      new KPropertiesDialog(item);
    }
  }
  else{
    if(currentItems){
      new KPropertiesDialog(*currentItems);
    }
  }
}

void MetabarWidget::slotShowSharingDialog()
{
  if(single_selection){
    if(!currentPath->isNull() && !currentPath->isEmpty()){
      KFileItem *item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      KPropertiesDialog *dialog = new KPropertiesDialog(item);
      dialog->showFileSharingPage();
    }
  }
}

void MetabarWidget::slotShowConfigDialog()
{
  config_dialog = new ConfigDialog(config, this);
  if(config_dialog->exec() == QDialog::Accepted){
    loadLinkList();
  
    if(single_selection){
      slotUpdateCurrentInfo(*currentPath);
    }
  }
  
  delete config_dialog;
}

void MetabarWidget::slotPopupInfo()
{
  const KURLLabel *title = static_cast<const KURLLabel*>(sender());
  QLabel *label = infoDict[title->text()];
  
  if(label){
    if(label->isVisible()){
      label->hide();
    }
    else{
      label->setMaximumWidth(info_frame->width() - 10);
      label->show();
    }
  }
}

void MetabarWidget::startPlayer()
{  
  if(playobj &&
     !playobj->object().isNull() &&
     playobj->state() != Arts::posPlaying)
  {
    
    setUpdatesEnabled(FALSE);
    
    if(isVideo){
      video->setFixedSize(preview->size());
      preview_stack->setMaximumHeight(preview->height());
      preview_stack->raiseWidget(video);
    }
    
    if(playobj->state() == Arts::posIdle){
      player_status->setValue(0);
      player_status->setMaxValue(playobj->overallTime().seconds);
    }
    
    player_timer->start(1000);
    playobj->play();
    
    setUpdatesEnabled(TRUE);
  }
}

void MetabarWidget::pausePlayer()
{  
  if(playobj &&
     !playobj->object().isNull() &&
     playobj->state() == Arts::posPlaying)
  {
    
    if(player_timer->isActive()){
      player_timer->stop();
    }
    
    playobj->pause();
  }
}

void MetabarWidget::stopPlayer()
{
  if(player_timer->isActive()){
    player_timer->stop();
  }
  player_status->setValue(0);
  
  if(playobj && 
    !playobj->object().isNull() &&
     playobj->state() != Arts::posIdle)
  {
    
    setUpdatesEnabled(FALSE);
    
    playobj->halt();
    
    video->setFixedSize(0, 0);
  
    if(isVideo){
      preview_stack->raiseWidget(preview);
    }
      
    setUpdatesEnabled(TRUE);
  }
}

void MetabarWidget::deletePlayer()
{  
  stopPlayer();
  
  playobj = 0;
  
  if(preview_stack->isVisible()){
    preview_stack->hide();
  }
}

void MetabarWidget::updatePlayerStatus()
{
  if(playobj && !playobj->object().isNull()){
    player_status->setValue(playobj->currentTime().seconds);
    
    if(playobj->currentTime().ms == playobj->overallTime().ms){
      player_timer->stop();
      stopPlayer();
    }
  }
}

void MetabarWidget::seekPlayer(int time)
{
  if(playobj && !playobj->object().isNull()){    
    playobj->seek(Arts::poTime(time, 0, 0, ""));
  }
}

QString MetabarWidget::getCurrentFolder()
{
  DCOPRef ref(kapp->dcopClient()->appId(), this->topLevelWidget()->name());
  DCOPReply reply = ref.call("currentURL()");
    
  if (reply.isValid()) {
    QString url;
    reply.get(url, "QString");
      
    if(!url.isNull() && !url.isEmpty()){
      return url;
    }
  }
  return 0;
}

void MetabarWidget::openURL(const QString &url)
{
  DCOPRef ref(kapp->dcopClient()->appId(), this->topLevelWidget()->name());
  DCOPReply reply = ref.call("openURL", url);
}

void MetabarWidget::openTab(const QString &url)
{
  DCOPRef ref(kapp->dcopClient()->appId(), this->topLevelWidget()->name());
  DCOPReply reply = ref.call("newTab", url);
}

void MetabarWidget::callAction(const QString &action)
{
  DCOPRef ref(kapp->dcopClient()->appId(), QString(this->topLevelWidget()->name()).append("/action/").append(action).utf8());
  ref.call("activate()");
}

void MetabarWidget::showEvent(QShowEvent*)
{
  *currentPath = QString("");
  
  KFileItemList list;
  setFileItems(list);
}

void MetabarWidget::resizeEvent(QResizeEvent*)
{
  QDictIterator<QLabel> it(infoDict);
  for(; it.current(); ++it){
    it.current()->setMaximumWidth(info_frame->width() - 10);
  }
}
  
void MetabarWidget::loadConfig()
{  
  config->setGroup("General");
  action_title->setOn(config->readBoolEntry("CollapseActions"));
  info_title->setOn(config->readBoolEntry("CollapseInfo"));
  preview_title->setOn(config->readBoolEntry("CollapsePreview"));
  link_title->setOn(config->readBoolEntry("CollapseLinks"));
}

void MetabarWidget::loadLinkList(){
  linkList.clear();
  
  config->setGroup("General");
  QStringList actions = config->readListEntry("Links");
  
  for(QStringList::Iterator it = actions.begin(); it != actions.end(); ++it){
    config->setGroup("Action_" + (*it));
    
    QString icon_str = config->readEntry("Icon", "folder");
    QPixmap icon(icon_str);
    if(icon.isNull()){
      icon = SmallIcon(icon_str);
    }
    
    LinkWidget *link = new LinkWidget(link_frame);
    link->setText(config->readEntry("Name"));
    link->setURL(config->readEntry("URL"));
    link->setPixmap(icon);
        
    connect(link, SIGNAL(leftClickedURL(const QString&)), this, SLOT(openURL(const QString&)));
    connect(link, SIGNAL(middleClickedURL(const QString&)), this, SLOT(openTab(const QString&)));
    
    if(!link_title->isOn()){
      link->show();
    }
    
    int index = config->readNumEntry("Index");
    if(index >= linkList.count()){
      index = -1;
    }
    else{
      index++;
    }
    
    link_frame_layout->insertWidget(index, link);
    linkList.append(link);
  }
}

#include "metabar_widget.moc"
